/*
    PYMAT -- A Python module that exposes the MATLAB engine interface
             and supports passing NumPy arrays back and forth.

  Revision History
  ----------------

  Version 1.0 -- December 26, 1998, Andrew Sterian (asterian@umich.edu)
     * Initial release
*/
/*
   PYOCT -- A Python Module interface to Octave
   Based on PYMAT 1.0 by Andrew Sterian
   
   Version 0.1 -- 4/3/99, Min Xu
*/
#define PYOCT_VERSION "0.1"
#include <Python.h>
#include <arrayobject.h>
#define OCT_ENGINE
#include "oct-engine.h"

#define AND &&
#define OR  ||
#define EQ  ==
#define NEQ !=

static void pyGenericError(PyObject *pException, const char *fmt, ...)
{
    char lBuf[1024];

    va_list ap;
    va_start(ap, fmt);
    vsprintf(& lBuf[0], fmt, ap);
    va_end(ap);
    
    strcat(&lBuf[0], "\n");

    PyErr_SetString(pException, & lBuf[0]);
}

/***************************************************/

static char open_doc[] = 
"handle open([str]) : Opens a Octave engine session.";

static PyObject *pyoct_open(PyObject *self, PyObject *args)
{
    Engine *ep;
    char *lStr = NULL;

    if (! PyArg_ParseTuple(args, "|s:open", &lStr)) return NULL;
    if ( sizeof(int) < sizeof(Engine *) ) {
	perror("Oops! Pointers on this architecture are too big to fit in integers");
	Py_INCREF(Py_None);
	return Py_None;
    }
	
    ep = engOpen(lStr);
    if (ep EQ NULL) {
        pyGenericError(PyExc_RuntimeError, "Unable to start Octave engine");
	return NULL;
    }

    return Py_BuildValue("i", (int)(ep));
}

static char close_doc[] = 
"void close(handle) : Closes Octave session\n";

static PyObject *pyoct_close(PyObject *self, PyObject *args)
{
    int lHandle;
    
    if (! PyArg_ParseTuple(args, "i:close", &lHandle)) return 0;
    
    if (engClose((Engine *)lHandle) NEQ 0) {
        pyGenericError(PyExc_RuntimeError, "Unable to close session");
        return NULL;
    }

    return Py_BuildValue("i", 0);
}

static char eval_doc[] = 
"void eval(handle, string) : Evaluates string in Octave session. Return 0 for success, NULL for failure.";

static PyObject *pyoct_eval(PyObject *self, PyObject *args)
{
    char *lStr;
    int lHandle;
    
    if (! PyArg_ParseTuple(args, "is:eval", &lHandle, &lStr)) return 0;

    if (engEvalString((Engine *)lHandle, lStr) NEQ 0) {
        pyGenericError(PyExc_RuntimeError, "Unable to evaluate string in Octave workspace");
        return 0;
    }

    return Py_BuildValue("i", 0);
}

static char get_doc[] = 
"array get(handle, name) : Gets a matrix from the Octave session.";

static PyObject *pyoct_get(PyObject *self, PyObject *args)
{
    char *lName;
    int lHandle;
    PyArrayObject *ap;

    if (! PyArg_ParseTuple(args, "is", &lHandle, &lName)) return 0;

    if ( (ap = engGetArray((Engine *)lHandle, lName))  EQ 0 ) {
        pyGenericError(PyExc_RuntimeError, "Unable to get matrix from Octave workspace");
	return 0;
    }
    if ( !PyArray_Check(ap) ) {
	pyGenericError(PyExc_RuntimeError, "Not an array object");
	return 0;
    }

    return PyArray_Return(ap);
}

static char put_doc[] = 
"void put(handle, name, array) : Places a matrix into the Octave session.";

static PyObject * pyoct_put(PyObject *self, PyObject *args)
{
    char *lName;
    int lHandle;
    PyArrayObject *lArray;

    if (! PyArg_ParseTuple(args, "isO:put", &lHandle, &lName, &lArray)) return 0;

    if (engPutArray((Engine *)lHandle, lName, lArray) NEQ 0) {
        pyGenericError(PyExc_RuntimeError, "Unable to put matrix into Octave workspace");
        return 0;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

static char echo_doc[] =
"void echo(size) echoes the output from Octave.\n";

static PyObject * pyoct_echo(PyObject *self, PyObject *args)
{
    int lHandle, size;
    char *buffer;
    Engine *ep;
    
    if (! PyArg_ParseTuple(args, "ii:echo", &lHandle, &size)) return 0;
    if (size < 0) return 0;

    ep = (Engine *)lHandle;
    if (size > 0) {
	buffer = (char *)malloc(size+1);
	engOutputBuffer(ep, buffer, size);
    } else {
	engOutputBuffer(ep, NULL, 0);
    }
    
    Py_INCREF(Py_None);
    return Py_None;
}

static PyMethodDef pyoctMethods[] = {
    { "open",       pyoct_open,       1, open_doc },
    { "close",      pyoct_close,      1, close_doc },
    { "eval",       pyoct_eval,       1, eval_doc },
    { "get",        pyoct_get,        1, get_doc },
    { "put",        pyoct_put,        1, put_doc },
    { "echo",       pyoct_echo,       1, echo_doc },
    { 0, 0}
};

void initpyoct(void)
{
    PyObject *module = Py_InitModule4("pyoct",
				      pyoctMethods,
"PyOct -- Octave Engine Interface\n"
"\n"
"  open  - Open a Octave engine session\n"
"  close - Close a Octave engine session\n"
"  eval  - Evaluates a string in the Octave session\n"
"  get   - Gets a matrix from the Octave session\n"
"  put   - Places a matrix into the Octave session\n"
"  echo  - Set the output buffer for Octave\n"
"\n"
"The NumPy Numeric package must be installed for this module to be used.\n"
,
				      0,
				      PYTHON_API_VERSION);

    PyObject *dict, *item;
    
    /* This macro, defined in arrayobject.h, loads the Numeric API interface */
    import_array();

    dict = PyModule_GetDict(module);
    item = PyString_FromString(PYOCT_VERSION);
    PyDict_SetItemString(dict, "__version__", item);
    Py_XDECREF(item);
}


